From: Bobby Eshleman <[email protected]>

Some virtual devices like netkit (or ifb) never DMA and never touch frag
contents, they just forward the skb to another device. They are unable
to forward unreadable skbs, however, because they fail to pass TX
validation checks on dev->netmem_tx. The existing two-state
NETMEM_TX_NONE / NETMEM_TX_DMA doesn't give the TX validator enough
information to differentiate devices that will attempt DMA on the
unreadable skb from those that will simply route it untouched.

Add a third mode to the enum so drivers can indicate 1) if they have
netmem TX support, and 2) if they do, whether they are DMA-capable:

NETMEM_TX_NO_DMA - pass-through, device never DMAs

Widen dev->netmem_tx from a 1-bit field to 2 bits to fit the new value,
and declare netkit as NETMEM_TX_NO_DMA. Devmem TX support over these
devices comes in a follow-up patch.

Signed-off-by: Bobby Eshleman <[email protected]>
---
Changes in v3:
- net_cachelines/net_device.rst: align the netmem_tx row's type column
  with the rest of the table by using "unsigned_long:2" instead of
  "unsigned long:2"
- Split this into a distinct patch (Jakub)
---
 Documentation/networking/net_cachelines/net_device.rst | 2 +-
 Documentation/networking/netmem.rst                    | 3 +++
 Documentation/translations/zh_CN/networking/netmem.rst | 3 +++
 drivers/net/netkit.c                                   | 1 +
 include/linux/netdevice.h                              | 7 ++++---
 5 files changed, 12 insertions(+), 4 deletions(-)

diff --git a/Documentation/networking/net_cachelines/net_device.rst 
b/Documentation/networking/net_cachelines/net_device.rst
index 1c19bb7705df..7b3392553fd6 100644
--- a/Documentation/networking/net_cachelines/net_device.rst
+++ b/Documentation/networking/net_cachelines/net_device.rst
@@ -10,7 +10,7 @@ Type                                Name                      
  fastpath_tx_acce
 =================================== =========================== 
=================== =================== 
===================================================================================
 unsigned_long:32                    priv_flags                  read_mostly    
                         __dev_queue_xmit(tx)
 unsigned_long:1                     lltx                        read_mostly    
                         HARD_TX_LOCK,HARD_TX_TRYLOCK,HARD_TX_UNLOCK(tx)
-unsigned long:1                     netmem_tx:1;                read_mostly
+unsigned_long:2                     netmem_tx:2;                read_mostly
 char                                name[16]
 struct netdev_name_node*            name_node
 struct dev_ifalias*                 ifalias
diff --git a/Documentation/networking/netmem.rst 
b/Documentation/networking/netmem.rst
index 5ccadba4f373..217869d1108d 100644
--- a/Documentation/networking/netmem.rst
+++ b/Documentation/networking/netmem.rst
@@ -99,3 +99,6 @@ Driver TX Requirements
    appropriate mode:
 
    - `NETMEM_TX_DMA`: for physical devices that perform DMA.
+
+   - `NETMEM_TX_NO_DMA`: for virtual or passthrough devices that do
+     not DMA, but still support handling of netmem-backed skbs.
diff --git a/Documentation/translations/zh_CN/networking/netmem.rst 
b/Documentation/translations/zh_CN/networking/netmem.rst
index 9c84423b7528..320f3eacf51b 100644
--- a/Documentation/translations/zh_CN/networking/netmem.rst
+++ b/Documentation/translations/zh_CN/networking/netmem.rst
@@ -92,3 +92,6 @@ dma-mapping API 去处理。
 2. 驱动程序应将 `netdev->netmem_tx` 设置为适当的模式:
 
    - `NETMEM_TX_DMA`:适用于执行 DMA 的物理设备。
+
+   - `NETMEM_TX_NO_DMA`:适用于不执行 DMA 的虚拟或透传设备,但仍支持
+     处理 netmem 支持的 skb。
diff --git a/drivers/net/netkit.c b/drivers/net/netkit.c
index 5e2eecc3165d..0ad6a806d7d5 100644
--- a/drivers/net/netkit.c
+++ b/drivers/net/netkit.c
@@ -466,6 +466,7 @@ static void netkit_setup(struct net_device *dev)
        dev->priv_flags |= IFF_NO_QUEUE;
        dev->priv_flags |= IFF_DISABLE_NETPOLL;
        dev->lltx = true;
+       dev->netmem_tx = NETMEM_TX_NO_DMA;
 
        dev->netdev_ops     = &netkit_netdev_ops;
        dev->ethtool_ops    = &netkit_ethtool_ops;
diff --git a/include/linux/netdevice.h b/include/linux/netdevice.h
index 580bccb118a0..11d68e75eb4f 100644
--- a/include/linux/netdevice.h
+++ b/include/linux/netdevice.h
@@ -1791,6 +1791,7 @@ enum netdev_stat_type {
 enum netmem_tx_mode {
        NETMEM_TX_NONE,         /* no netmem TX support */
        NETMEM_TX_DMA,          /* DMA-capable netmem TX (real HW) */
+       NETMEM_TX_NO_DMA,       /* no DMA, e.g. passthrough for virtual devs */
 };
 
 enum netdev_reg_state {
@@ -1814,8 +1815,8 @@ enum netdev_reg_state {
  *     @lltx:          device supports lockless Tx. Deprecated for real HW
  *                     drivers. Mainly used by logical interfaces, such as
  *                     bonding and tunnels
  *     @netmem_tx:     device netmem TX mode (NETMEM_TX_NONE or
  *                     NETMEM_TX_DMA).
  *
  *     @name:  This is the first field of the "visible" part of this structure
  *             (i.e. as seen by users in the "Space.c" file).  It is the name
@@ -2138,7 +2139,7 @@ struct net_device {
        struct_group(priv_flags_fast,
                unsigned long           priv_flags:32;
                unsigned long           lltx:1;
-               unsigned long           netmem_tx:1;
+               unsigned long           netmem_tx:2;
        );
        const struct net_device_ops *netdev_ops;
        const struct header_ops *header_ops;
@@ -1814,8 +1815,8 @@ enum netdev_reg_state {
  *                     drivers. Mainly used by logical interfaces, such as
  *                     bonding and tunnels
- *     @netmem_tx:     device netmem TX mode (NETMEM_TX_NONE or
- *                     NETMEM_TX_DMA).
+ *     @netmem_tx:     device netmem TX mode (NETMEM_TX_NONE, NETMEM_TX_DMA,
+ *                     or NETMEM_TX_NO_DMA).
  *
  *     @name:  This is the first field of the "visible" part of this structure

-- 
2.53.0-Meta


Reply via email to